@@ -17,11 +17,12 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 """A simple implementation of a Message Catalog.
 
-$Id: GettextMessageCatalog.py,v 1.24.2.1 2004/11/25 22:17:40 longsleep Exp $
+$Id: GettextMessageCatalog.py 10622 2005-08-07 19:10:13Z batlogg $
 """
 
 from gettext import GNUTranslations
 import os, sys, types, codecs, traceback, time
+import glob
 import re
 
 from Acquisition import aq_parent, Implicit
@@ -29,6 +30,7 @@ from DateTime import DateTime
 from AccessControl import ClassSecurityInfo
 from AccessControl.Permissions import view, view_management_screens
 from Globals import InitializeClass
+from Globals import INSTANCE_HOME
 import Globals
 from OFS.Traversable import Traversable
 from Persistence import Persistent, Overridable
@@ -68,6 +70,8 @@ permission = 'View management screens'
 
 translationRegistry = Registry()
 registerTranslation = translationRegistry.register
+rtlRegistry = Registry()
+registerRTL = rtlRegistry.register
 
 def getMessage(catalog, id, orig_text=None):
     """get message from catalog
@@ -234,7 +238,19 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
             self.preferred_encodings = tro._info.get('preferred-encodings', '').split()
             self.name = unicode(tro._info.get('language-name', ''), tro._charset)
             self.default_zope_data_encoding = tro._charset
+
             translationRegistry[self.getId()] = self._v_tro = tro
+
+            # right to left support
+            is_rtl = tro._info.get('x-is-rtl', 'no').strip().lower()
+            if is_rtl in ('yes', 'y', 'true', '1'):
+                self._is_rtl = True
+            elif is_rtl in ('no', 'n', 'false', '0'):
+                self._is_rtl = False
+            else:
+                raise ValueError, 'Unsupported value for X-Is-RTL' % is_rtl
+            rtlRegistry[self.getId()] = self.isRTL()
+
             missingFileName = self._pofile[:-3] + '.missing'
             if os.access(missingFileName, os.W_OK):
                 tro.add_fallback(MissingIds(missingFileName, self._v_tro._charset))
@@ -317,6 +333,11 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
         """
         self._prepareTranslations()
         return self._v_tro._info.get(name, None)
+    
+    def isRTL(self):
+        """
+        """
+        return self._is_rtl
 
     security.declareProtected(view_management_screens, 'Title')
     def Title(self):
@@ -325,8 +346,13 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
     def _getMoFile(self):
         """get compiled version of the po file as file object
         """
-        mo = Msgfmt(self._readFile(), self.getId())
-        return mo.getAsFile()
+        useCache = True
+        if useCache:
+            hit, mof = cachedPoFile(self)
+            return mof
+        else:
+            mo = Msgfmt(self._readFile(), self.getId())
+            return mo.getAsFile()
 
     def _readFile(self, reparse=False):
         """Read the data from the filesystem.
@@ -583,3 +609,128 @@ class MissingIds(Persistent):
         raise KeyError, msgid
 
 InitializeClass(MissingIds)
+
+class MoFileCache(object):
+    """Cache for mo files
+    """
+    
+    def __init__(self, path):
+        if not os.path.isdir(path):
+            try:
+                os.makedirs(path)
+            except (IOError, OSError):
+                path = None
+                log("No permission to create directory %s" % path, PROBLEM)
+        self._path = path
+        
+    def storeMoFile(self, catalog):
+        """compile and save to mo file for catalog to disk
+        
+        return value: mo file as file handler
+        """
+        log('Storing mo file for %s' % catalog.getId(), severity=BLATHER)
+        f = self.getPath(catalog)
+        mof = self.compilePo(catalog)
+        moExists = os.path.exists(f)
+        if (not moExists and os.access(self._path, os.W_OK)) \
+          or (moExists and os.access(f, os.W_OK)):
+            fd = open(f, 'wb')
+            fd.write(mof.read()) # XXX efficient?
+            fd.close()
+        else:
+            log("No permission to write file %s" % f, PROBLEM)
+        mof.seek(0)
+        return mof
+        
+    def retrieveMoFile(self, catalog):
+        """Load a mo file file for a catalog from disk
+        """
+        log('Cache hit for %s' % catalog.getId(), severity=BLATHER)
+        f = self.getPath(catalog)
+        if os.path.isfile(f):
+            if os.access(f, os.R_OK):
+                return open(f, 'rb')
+            else:
+                log("No permission to read file %s" % f, PROBLEM)
+                return None
+        
+    def getPath(self, catalog):
+        """Get the mo file path (cache path + file name)
+        """
+        id = catalog.getId()
+        if id.endswith('.po'):
+            id = id[:-3]
+        return os.path.join(self._path, '%s.mo' % id)
+        
+    def isCacheHit(self, catalog):
+        """Cache hit?
+        
+        True: file exists and mod time is newer than mod time of the catalog
+        False: file exists but mod time is older
+        None: file doesn't exist
+        """
+        f = self.getPath(catalog)
+        ca_mtime = catalog._getModTime()
+        try:
+            mo_mtime = os.stat(f)[8]
+        except (IOError, OSError):
+            mo_mtime = 0
+        
+        if mo_mtime == 0:
+            return None
+        elif ca_mtime == 0:
+            return None
+        elif mo_mtime > ca_mtime:
+            return True
+        else:
+            return False
+        
+    def compilePo(self, catalog):
+        """compile a po file to mo
+        
+        returns a file handler
+        """
+        mo = Msgfmt(catalog._readFile(), catalog.getId())
+        return mo.getAsFile()
+        
+    def cachedPoFile(self, catalog):
+        """Cache a po file (public api)
+        
+        Returns a file handler on a mo file
+        """
+        path = self._path
+        if path is None:
+            return None, self.compilePo(catalog)
+        hit = self.isCacheHit(catalog)
+        if hit:
+            mof = self.retrieveMoFile(catalog)
+            if mof is None:
+                mof = self.compilePo(catalog)
+        else:
+            mof = self.storeMoFile(catalog)
+        return hit, mof
+        
+    def purgeCache(self):
+        """Purge the cache and remove all compiled mo files
+        """
+        log("Purging mo file cache", INFO)
+        if not os.access(self._path, os.W_OK):
+            log("No write permission on folder %s" % self._path, PROBLEM)
+            return False
+        pattern = os.path.join(self._path, '*.mo')
+        for mo in glob.glob(pattern):
+            if not os.access(mo, os.W_OK):
+                log("No write permission on file %s" % mo, PROBLEM)
+                continue
+            #log("Removing mo file %s" % mo, INFO)
+            try:
+                os.unlink(mo)
+            except IOError:
+                log("Failed to unlink %s" % mo, PROBLEM)
+
+
+_moCache = MoFileCache(os.path.join(INSTANCE_HOME, 'var', 'pts'))
+cachedPoFile = _moCache.cachedPoFile
+purgeMoFileCache = _moCache.purgeCache
+
+    
